/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XHTML + Voice support.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Chris Cross <xcross@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#if !defined(SMSCOPE_H)
#define SMSCOPE_H

#include "jsapi.h"
#include "nsVoidArray.h"
#include "PLEvent.h"
#include "vxmscope.h"
#include "nsIEventQueue.h"
#include "nsIEventQueueService.h"
#include "nsIDocument.h"

/**
 * Spider Monkey Object node
 */
class SMObject
{
public:
	SMObject(JSObject* object)
		: m_object(object) {}
	~SMObject() {}
	JSObject* m_object;
};

class MozillaScope;

struct InitScopeEvent : public PLEvent 
{
	InitScopeEvent(nsIDocument *aDoc): mDoc(aDoc),  mRc(VXM_RC_OK) {}
	nsIDocument *mDoc;
	VxmRc mRc;
};

struct MozillaScopeEvent : public PLEvent 
{
	MozillaScopeEvent(MozillaScope * aScope) : mScope(aScope), mRc(VXM_RC_OK) {}
	MozillaScope * mScope;
	VxmRc mRc;
};

struct JSEvalEvent : public MozillaScopeEvent
{
	JSEvalEvent(MozillaScope * aScope, wchar_t *script, EcmaScriptValue* result, bool bReleaseObject) 
		: MozillaScopeEvent(aScope), mResult(result), mReleaseObject(bReleaseObject)
	{mScript = script;}

	nsString mScript;
	EcmaScriptValue* mResult;
	bool mReleaseObject;
};

struct JSLeaveEvent : public MozillaScopeEvent
{
	JSLeaveEvent(MozillaScope * aScope) 
		: MozillaScopeEvent(aScope){}
};

struct JSEnterEvent : public MozillaScopeEvent
{
	JSEnterEvent(MozillaScope * aScope, wchar_t *aName, ESObject* aObject)
		: MozillaScopeEvent(aScope), mObject(aObject)
	{mName = aName;}

	nsString mName;
	ESObject* mObject;
};

struct JSSetSlotEvent : public MozillaScopeEvent
{
	JSSetSlotEvent(MozillaScope * aScope, const wchar_t * aSlotName, const EcmaScriptValue& aValue)
		: MozillaScopeEvent(aScope), mValue(aValue)
	{mSlotName = aSlotName;}

	nsString mSlotName;
	EcmaScriptValue mValue;
};

struct JSGetSlotEvent : public MozillaScopeEvent
{
	JSGetSlotEvent(MozillaScope * aScope, const wchar_t * aSlotName, EcmaScriptValue * aValue)
		: MozillaScopeEvent(aScope), mValue(aValue)
	{mSlotName = aSlotName;}

	nsString mSlotName;
	EcmaScriptValue * mValue;
};

struct JSSetSlotObjectEvent : public MozillaScopeEvent
{
	JSSetSlotObjectEvent(MozillaScope * aScope, const wchar_t * aSlotName, const EcmaScriptValue& aValue, ESObject aObject)
		: MozillaScopeEvent(aScope), mValue(aValue), mObject(aObject)
	{mSlotName = aSlotName;}

	nsString mSlotName;
	EcmaScriptValue mValue;
	ESObject mObject;
};

struct JSGetSlotObjectEvent : public MozillaScopeEvent
{
	JSGetSlotObjectEvent(MozillaScope * aScope, const wchar_t * aSlotName, EcmaScriptValue * aValue, ESObject aObject)
		: MozillaScopeEvent(aScope), mValue(aValue), mObject(aObject)
	{mSlotName = aSlotName;}

	nsString mSlotName;
	EcmaScriptValue * mValue;
	ESObject mObject;
};

struct JSGetElementEvent : public MozillaScopeEvent
{
	JSGetElementEvent(MozillaScope * aScope, int index, EcmaScriptValue* aValue, ESObject aArray)
		: MozillaScopeEvent(aScope), mIndex(index), mValue(aValue), mArray(aArray){}

	int mIndex;
	EcmaScriptValue * mValue;
	ESObject mArray;
};

struct JSSetElementEvent : public MozillaScopeEvent
{
	JSSetElementEvent(MozillaScope * aScope, int index, const EcmaScriptValue&  aValue, ESObject aArray)
		: MozillaScopeEvent(aScope), mIndex(index), mValue(aValue), mArray(aArray){}

	int mIndex;
	const EcmaScriptValue&  mValue;
	ESObject mArray;
};

struct JSGetRecordingEvent : public MozillaScopeEvent
{
	JSGetRecordingEvent(MozillaScope * aScope, const EcmaScriptValue& aValue, wchar_t** recHandle)
		: MozillaScopeEvent(aScope), mValue(aValue), mhRecording(recHandle){}

	const EcmaScriptValue& mValue;
	wchar_t ** mhRecording;
};

struct JSSetRecordingEvent : public MozillaScopeEvent
{
	JSSetRecordingEvent(MozillaScope * aScope, const wchar_t * slotName, const wchar_t * recHandle)
		: MozillaScopeEvent(aScope), mSlotName(slotName), mhRecording(recHandle){}

	const wchar_t * mSlotName;
	const wchar_t * mhRecording;
};

struct JSCreateObjectEvent : public MozillaScopeEvent
{
	JSCreateObjectEvent(MozillaScope * aScope, ESObject* aObject)
		: MozillaScopeEvent(aScope), mObject(aObject){}

	ESObject* mObject;
};

struct JSCreateArrayObjectEvent : public MozillaScopeEvent
{
	JSCreateArrayObjectEvent(MozillaScope * aScope, ESObject* array)
		: MozillaScopeEvent(aScope), mArray(array){}

	ESObject* mArray;
};

/**
 * Mozilla Scope class
 */
class MozillaScope: public VxmScope
{
private:
	/**Ctor
	 * @param context pointer to Spider Monkey thread context
	 * @param aDoc  pointer to the document
	 */
	MozillaScope(JSContext* context, nsIDocument* aDoc);

public:
	/** Factory method used as callback by the VXM to create new scopes in the chain.
	 *  @param context (in) pointer to native JSContext 
	 *  @param globalScope (in) pointer to nsIDocument 
	 *  @return new VxmScope pointer, NULL if failed
	 */
	static VxmScope * createScope(VxmContext context, GlobalScope globalScope);

	/**Dtor
	 */
	~MozillaScope();	
    /**Evaluate the EcmaScript contained in script as program text.
     *  Return the result of the evaluation in the result.
     * @param script (in) Program text to compile and execute.
     * @param result (out) Result of script evaluation.  Input to result
	  *                     is NULL if caller not interested in it.
	  * @param bReleaseObject (in) if true release object if object returned
	  * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED
     */
	virtual VxmRc eval(const wchar_t * script, EcmaScriptValue * result, bool bReleaseObject);
    /**Enter a new inner scope
     * @param name (in) Name of the scope object
	 * @param object (out) scope object created by enter.  Input to object
	 *                     is NULL if caller not interested in it.
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED
     */
	virtual VxmRc enter(const wchar_t * name, ESObject* object);
    /**Leave the current scope
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_SCOPE_RC_NO_OBJECT
     */
	virtual VxmRc leave();
	/**Create an unnamed object
	 * @param object (out) handle to the created object.  This
	 *                  object must be later released by
	 *                  calling releaseObject
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED,
	 *         VXMI_SCOPE_RC_NO_OBJECT
	 */
	virtual VxmRc createObject(ESObject* object);
	/**Create an unnamed array
	 * @param array (out) handle to the created array.  This
	 *                    array must be later released by
	 *                    calling releaseObject.
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED,
	 *         VXMI_SCOPE_RC_NO_OBJECT
	 */
	virtual VxmRc createArrayObject(ESObject* array);
	/**Release an object or array
	 * @param object (in) handle to the object
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_SCOPE_RC_NO_OBJECT
	 */
	virtual VxmRc releaseObject(ESObject object);
    /**Get the value of a field within an object
     * @param slotName (in) Name of the field
     * @param result (out) returned value of the field
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED,
	 *         VXMI_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc getSlot(const wchar_t * slotName, EcmaScriptValue* result);
    /**Get the value of a field within an object
     * @param slotName (in) Name of the field
     * @param result (out) returned value of the field
	 * @param object (in) handle to the slot's object.
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED
     */
    virtual VxmRc getSlot(const wchar_t * slotName, EcmaScriptValue* result, ESObject object);
    /**Get the value of an element within an array
     * @param index (in) index number of the element in array
     * @param result (out) returned value of the field
	 * @param array (in) handle to the element's array
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED,
	 *         VXMI_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc getElement(int index, EcmaScriptValue* result, ESObject array);
	/**Get the recording value of a field within an object
	 * @param value (in) the audio recording container
	 * @param recHandle (out) returned recording handle of the field, or empty
	 *                        if not a recording
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED
	 */
	virtual VxmRc getRecording(const EcmaScriptValue& value, wchar_t ** recHandle);
    /**Set the value of a field within an object
     * @param slotName (in) name of the field
     * @param value (in) the new value of the field
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED,
	 *         VXMI_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc setSlot(const wchar_t *slotName, const EcmaScriptValue& value);
    /**Set the value of a field within an object
     * @param slotName (in) name of the field
     * @param value (in) the new value of the field
	 * @param object (in) handle to the parent object.
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED
     */
    virtual VxmRc setSlot(const wchar_t *slotName, const EcmaScriptValue& value, ESObject object);
    /**Set the value of a field within an object
     * @param index (in) index number of the element in array
     * @param value (in) The new value of the field
	 * @param array (in) handle to the element's array
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED,
	 *         VXMI_SCOPE_RC_NO_OBJECT
     */
    virtual VxmRc setElement(int index, const EcmaScriptValue& value, ESObject array);
	/**Set the value of a field within an object to be a handle to a recording
	 * @param slotName (in) name of the field
	 * @param recHandle (in) returned recording handle of the field
	 * @return VXMI_RC_OK, VXMI_RC_INTERNAL_ERROR, VXMI_RC_CANCELED
	 */
	virtual VxmRc setRecording(const wchar_t* slotName, const wchar_t * recHandle);
	/**Delete a string that was created from a previous eval(), getSlot() or getElement() call
	 * @param ptrString (in) pointer to the string to free
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR, VXM_RC_CANCELED
	 */
	virtual VxmRc freeString(wchar_t * ptrString);

	/**Initialize MozillaScope
	 * @param aDoc (in) pointer to nsIDocument
	 * @return VXM_RC_OK, VXM_RC_INTERNAL_ERROR
	 */
 	static VxmRc Init(nsIDocument *aDoc);

  /**
   */
  static void Shutdown();

private: // member data
	nsCOMPtr<nsIDocument> m_Doc;
  JSContext *m_Context;             // Javascript context
	JSObject  *m_Object;              // Global Javascript object
	nsAutoVoidArray m_scopeChain;     // EcmaScript object scope chain
	static int m_errorRc;             // Return code obtained from ErrorReporter

	static nsAutoVoidArray gDocScopes;



private: //Mozilla Event Queue methods
//These are implemented to forward Javascript operations to the UI thread
//from the thread that is running nsVxmInterpreter::RunHandler

	static void * HandleInitEvent(PLEvent *evt);

	static void * HandleGetElementEvent(PLEvent *evt);
	static void * HandleSetElementEvent(PLEvent *evt);
	static void * HandleGetRecordingEvent(PLEvent *evt);
	static void * HandleSetRecordingEvent(PLEvent *evt);

	static void * HandleEvalEvent(PLEvent *evt);
	static void * HandleEnterEvent(PLEvent *evt);
	static void * HandleLeaveEvent(PLEvent *evt);
	static void * HandleSetSlotEvent(PLEvent *evt);
	static void * HandleGetSlotEvent(PLEvent *evt);
	static void * HandleSetSlotObjectEvent(PLEvent *evt);
	static void * HandleGetSlotObjectEvent(PLEvent *evt);
	static void * HandleCreateObjectEvent(PLEvent *evt);
	static void * HandleCreateArrayObjectEvent(PLEvent *evt);

	static void DestroyInitEvent(PLEvent *evt);

	static void DestroyEvalEvent(PLEvent *evt);
	static void DestroyEnterEvent(PLEvent *evt);
	static void DestroyGetSlotEvent(PLEvent *evt);
	static void DestroySetSlotEvent(PLEvent *evt);
	static void DestroyGetSlotObjectEvent(PLEvent *evt);
	static void DestroySetSlotObjectEvent(PLEvent *evt);
	static void DestroyCreateObjectEvent(PLEvent *evt);
	static void DestroyCreateArrayObjectEvent(PLEvent *evt);
	static void DestroyLeaveEvent(PLEvent *evt);
	static void DestroyGetElementEvent(PLEvent *evt);
	static void DestroySetElementEvent(PLEvent *evt);
	static void DestroyGetRecordingEvent(PLEvent *evt);
	static void DestroySetRecordingEvent(PLEvent *evt);

	/**Error Reporter handler routine for SpiderMonkey errors
	 * @param context The SpiderMonkey context
	 * @param msg     error message character buffer
	 * @param report  SpiderMonkey JSErrorReport structure
	 */
	static void errorReporter(JSContext* context, const char* msg, JSErrorReport* report);
};

#endif //!defined(SMSCOPE_H)
