/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XHTML + Voice support.
 *
 * The Initial Developer of the Original Code is IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Chris Cross <xcross@us.ibm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include <stdio.h>
#include "nsString.h"
#include "nsVxmlElementFactory.h"

NS_IMPL_ISUPPORTS2(nsVxmlElementFactory, 
                   nsIXTFElementFactory,
                   nsIDOMNSFeatureFactory)

NS_HIDDEN_(nsresult) NS_NewVxmlElement(nsIXTFElement **aResult);
NS_HIDDEN_(nsresult) NS_NewVxmlFormElement(nsIXTFElement **aResult);
NS_HIDDEN_(nsresult) NS_NewXvHandlerElement(nsIXTFElement **aResult);

NS_IMETHODIMP
nsVxmlElementFactory::CreateElement(const nsAString& aTagName,
                                      nsIXTFElement **aElement)
{

  //wprintf(L"element: %s\n", aTagName.BeginReading());

  if (aTagName.EqualsLiteral("form"))
  {
    //__asm {int 3}
    return NS_NewVxmlFormElement(aElement);
  }
  if (aTagName.EqualsLiteral("assign"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("audio"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("block"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("break"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("catch"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("clear"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("desc"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("else"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("elseif"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("emphasis"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("enumerate"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("error"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("example"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("field"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("filled"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("form"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("grammar"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("help"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("if"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("initial"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("item"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("lexicon"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("log"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("mark"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("meta"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("metadata"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("noinput"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("nomatch"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("one-of"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("option"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("p"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("param"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("phoneme"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("prompt"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("property"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("prosody"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("record"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("reprompt"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("return"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("rule"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("ruleref"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("s"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("say-as  "))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("sub"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("subdialog"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("tag"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("throw"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("token"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("value"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("var"))
	  return NS_NewVxmlElement(aElement);
  if (aTagName.EqualsLiteral("voice"))
	  return NS_NewVxmlElement(aElement);

  if (aTagName.EqualsLiteral("handler"))
	  return NS_NewXvHandlerElement(aElement);

  *aElement = nsnull;
  return NS_ERROR_FAILURE;
}

NS_IMETHODIMP
nsVxmlElementFactory::HasFeature(nsISupports *aObject,
                                   const nsAString& aFeature,
                                   const nsAString& aVersion,
                                   PRBool *aReturn)
{
	return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
nsVxmlElementFactory::GetFeature(nsISupports *aObject,
                                   const nsAString & aFeature,
                                   const nsAString & aVersion,
                                   nsISupports **aReturn)
{
	return NS_ERROR_NOT_IMPLEMENTED;
}

